// usb2lpt.c   V3.15.3
// THESE FUNCTIONS PERFORM THE IO TO THE USB2LPT 1.6 DEVICE
// based on V-USB driver by OBJECTIVE DEVELOPMENT Software GmbH
// see: http://www.obdev.at/products/vusb/index.html
// and  http://vusb.wikidot.com/
// and  the firmware usb2lpt6.c for USB2LPT 1.6 by Henrik Haftmann 
// see  http://www-user.tu-chemnitz.de/~heha/bastelecke/Rund%20um%20den%20PC/USB2LPT/index.html.en
// Pierre Vanhoucke / ON5GN

#if LIBUSB_PRESENT == 1

#include <usb.h>  
#include <string.h>  
#include "globdef.h"
#include "uidef.h"
#include "screendef.h"

#define USBDEV_SHARED_VENDOR    0x16C0   // VOTI    VID
#define USBDEV_SHARED_PRODUCT   0x06B3   // USB2LPT PID 
#define USB_ERROR_NOTFOUND  1
#define USB_ERROR_ACCESS    2
#define USB_ERROR_IO        3
#define MAX_USB_ERR_CNT     6
#define DDR_OUT 0x00FF

int USB2LPT_open_switch = 0;
usb_dev_handle *USB2LPT_handle;

void open_USB2LPT(void)
{  

void out_firmware (unsigned char port, unsigned char data) 
{ 
int nBytes;
unsigned char   in_byte[1];
int err_cnt; 

err_cnt =0;   
out_again:;    
nBytes = usb_control_msg
  (
  USB2LPT_handle, 
  USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, // request-type
  0x92,             //  request_id parameter
  port + data*256,  //  value parameter: first byte = portnumber, second byte = data
  0,                //  index parameter 
  (char *)in_byte,  //  pointer to  buffer
  sizeof(in_byte), 
  5000);            //  timeoutInMilliseconds
if (nBytes != 0)
  {
  err_cnt ++;
  if(err_cnt < MAX_USB_ERR_CNT)
    {
    lir_sleep(1000); // settling time
    goto out_again; 
    }
  else
    {
    printf("usb2lpt.c: Error during firmware setup \n"); 
    printf("usb error message: %s\n", usb_strerror());
    USB2LPT_open_switch = 0;
    }
  }
}  

int  usbGetStringAscii(usb_dev_handle *dev, int our_index, int langid, char *buf, int buflen)
{
char    buffer[256];
int     rval, i;

if((rval = usb_control_msg
   (
   dev,
   USB_ENDPOINT_IN,
   USB_REQ_GET_DESCRIPTOR,
   (USB_DT_STRING << 8) + our_index,
   langid,
   buffer,
   sizeof(buffer),
   1000))   < 0) return rval;
if(buffer[1] != USB_DT_STRING) return 0;
if((unsigned char)buffer[0] < rval)  rval = (unsigned char)buffer[0];
rval /= 2;
// lossy conversion to ISO Latin1 
for(i=1;i<rval;i++)
  {
  if(i > buflen)  break;                      // destination buffer overflow 
  buf[i-1] = buffer[2 * i];
  if(buffer[2 * i + 1] != 0)  buf[i-1] = '?'; // outside of ISO Latin1 range 
  }
buf[i-1] = 0;
return i-1;
}

int usbOpenDevice(usb_dev_handle **device, int vendor, char *vendorName, int product, char *productName)
{
struct usb_bus      *bus;
struct usb_device   *dev;
int                 errorCode = USB_ERROR_NOTFOUND;
static int          didUsbInit = 0;
char    string[256];
int     len;

if(!didUsbInit)
  {
  didUsbInit = 1;
  usb_init();
  }
usb_find_busses();
usb_find_devices();
for(bus=usb_get_busses(); bus; bus=bus->next)
  {
  for(dev=bus->devices; dev; dev=dev->next)
    {
    if(dev->descriptor.idVendor == vendor && dev->descriptor.idProduct == product)
      {
      USB2LPT_handle = usb_open(dev); // we need to open the device in order to query strings 
      if(!USB2LPT_handle)
        {
        errorCode = USB_ERROR_ACCESS;
        printf("usb2lpt.c: Warning: cannot open USB device. \n");
        printf("usb error message:%s\n", usb_strerror());
        continue;
        }
      if(vendorName == NULL && productName == NULL) break;  // name does not matter 
// now check whether the names match:
      len = usbGetStringAscii(USB2LPT_handle, dev->descriptor.iManufacturer, 0x0409, string, sizeof(string));
      if(len < 0)
        {
        errorCode = USB_ERROR_IO;
        printf("usb2lpt.c: Warning: cannot query manufacturer for device.\n");
        printf("usb error message:%s\n", usb_strerror());
        }
      else
        {
        errorCode = USB_ERROR_NOTFOUND;
        // printf("seen device from vendor ->%s<-\n", string); 
        if(strcmp(string, vendorName) == 0)
          {
          len = usbGetStringAscii(USB2LPT_handle, dev->descriptor.iProduct, 0x0409, string, sizeof(string));
          if(len < 0)
            {
            errorCode = USB_ERROR_IO;
            printf("usb2lpt.c: Warning: cannot query product for device.\n");
            printf("usb error message:%s\n", usb_strerror());
            }
          else
            {
            errorCode = USB_ERROR_NOTFOUND;
            // printf("seen product ->%s<-\n", string); 
            if(strcmp(string, productName) == 0) break;
            }
          }
        }
      usb_close(USB2LPT_handle);
      USB2LPT_handle = NULL;
      }
    }
  if(USB2LPT_handle)  break;
  }
if(USB2LPT_handle != NULL)
  {
  errorCode = 0;
  *device = USB2LPT_handle;
  }
return errorCode;
}

if (USB2LPT_open_switch == 0)
  {
  int retval;
  char s[80];
  double dt1;

  USB2LPT_handle = NULL;
  retval=(usbOpenDevice(&USB2LPT_handle,
                        USBDEV_SHARED_VENDOR,
                        "haftmann#software",
                        USBDEV_SHARED_PRODUCT,
                        "USB2LPT low-speed adapter") );
  if(retval != 0 )
    {
    printf("usb2lpt.c: Could not find USB device \"USB2LPT low-speed adapter\" with vid=0x%x pid=0x%x, retrying.\n",
                 USBDEV_SHARED_VENDOR,
                 USBDEV_SHARED_PRODUCT);
    dt1=current_time();
    while(retval != 0 )
      {
      settextcolor(12);
      sprintf(s,"No USB2LPT 1.6 device found, please connect your device: Waiting %.2f",
                                                                      current_time()-dt1);
      lir_text(0,screen_last_line,s);
      settextcolor(7);
      lir_refresh_screen();
      if(kill_all_flag)return; 
      lir_sleep(30000);            
      retval=(usbOpenDevice(&USB2LPT_handle,
                            USBDEV_SHARED_VENDOR,
                            "haftmann#software",
                            USBDEV_SHARED_PRODUCT,
                            "USB2LPT low-speed adapter") );
      }
    clear_lines ( screen_last_line, screen_last_line );
    lir_refresh_screen();
    printf("usb2lpt.c: Retry successful, USB device \"USB2LPT low-speed adapter\" with vid=0x%x pid=0x%x found.\n",
                 USBDEV_SHARED_VENDOR,
                 USBDEV_SHARED_PRODUCT);
    }
  USB2LPT_open_switch = 1;
//issue setup-commands for the USB2LPT 1.6 firmware 
  out_firmware(0x0F,0x00);	            	// Normal operation: Pullups on
  out_firmware(0x0A,0x00);	             	// SPP-Modus
  out_firmware(0x0D,(unsigned char)~DDR_OUT);   // Statusport      = Input  
  out_firmware(0x0C,DDR_OUT);     	     	// Dataport        = Output
  out_firmware(0x0E,DDR_OUT);	     	        // Controlport	   = Output
  }
}

void out_USB2LPT (unsigned char data, unsigned char port) 
{ 
int nBytes;
unsigned char   in_byte[1];
int err_cnt;
char s[80];

open_USB2LPT();  
err_cnt =0;   
out_again:;       
nBytes = usb_control_msg
  (
  USB2LPT_handle, 
  USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, // request-type
  0x92,             //  request_id parameter
  port + data*256,  //  value parameter:  first byte = portnumber, second byte = data
  0,                //  index parameter 
  (char *)in_byte,  //  pointer to  buffer
  sizeof(in_byte), 
  5000);            //  timeoutInMilliseconds
if (nBytes != 0)
  {
  err_cnt ++;
  if(err_cnt < MAX_USB_ERR_CNT)
    {
    lir_sleep(1000); // settling time
    goto out_again; 
    }
  else
    {
    printf("usb2lpt.c: Error when writing to port=%i, data=%#4.2x, returncode=%i.\n",port,data,nBytes); 
    printf("usb error message: %s\n", usb_strerror());
    settextcolor(12);
    sprintf(s,"               Error when writing to usb2lpt device.   ");
    lir_text(0,screen_last_line,s);
    settextcolor(7);
    lir_refresh_screen();
    }
  }
//printf("                 OUT_USB  port= %i  data=%#4.2x \n",port,data); 
//fflush(stdout); 
return ;
} 

unsigned char in_USB2LPT( unsigned char port)
{    
int nBytes;
unsigned char   in_byte[1] ;           
int err_cnt;
char s[80];

open_USB2LPT();
err_cnt =0;
in_again:;
in_byte[0] = 0;
nBytes     = 0;
nBytes = usb_control_msg
  (
  USB2LPT_handle, 
  USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, // request-type
  0x01,   		// request_id parameter
  port, 		// value parameter
  0,      		// index parameter 
  (char *)in_byte,      // pointer to buffer
  sizeof(in_byte), 
  5000);                // timeoutInMilliseconds
if (nBytes!= 1)         // error recovery
  {
  err_cnt ++;
  if(err_cnt < MAX_USB_ERR_CNT)
    {
    lir_sleep(1000);  	// settling time
    goto in_again; 
    }
  else
    {
    printf("usb2lpt.c: Error when reading from port=%i, data=%#4.2x, returncode=%i.\n",port,in_byte[0],nBytes); 
    printf("usb error message: %s\n", usb_strerror());
    in_byte[0] = 0;
    settextcolor(12);
    sprintf(s,"               Error when reading from usb2lpt device.   ");
    lir_text(0,screen_last_line,s);
    settextcolor(7);
    lir_refresh_screen();
    }
  }
//printf("IN_USB  port= %i  data=%#4.2x \n",port,in_byte[0] );
//fflush(stdout); 
return in_byte[0] ;
}

#endif
